@@ -21,7 +21,7 @@
 #' @param expand \code{[logical]}. If \code{TRUE}, \code{id1} gains new records if \code{id2} indicates a match. \emph{Not interchangeable with \code{shrink}}.
 #' @param shrink \code{[logical]}. If \code{TRUE}, \code{id1} loses existing records \code{id2} does not indicate a match. \emph{Not interchangeable with \code{expand}}.
 #'
-#' @seealso \code{\link{links}}; \code{\link{links_sv_probabilistic}}
+#' @seealso \code{\link{links}}; \code{\link{links_af_probabilistic}}
 #'
 #' @examples
 #' id1 <- rep(1, 5)
@@ -50,6 +50,9 @@ merge_ids <- function(...) UseMethod("merge_ids")
 #' @export
 merge_ids.default <- function(id1, id2, tie_sort = NULL,
                               expand = TRUE, shrink = FALSE, ...){
+  #
+  overwrite <- FALSE
+  #
   err <- err_atomic_vectors(id1, "id1")
   if(err != FALSE) stop(err, call. = FALSE)
   err <- err_atomic_vectors(id2, "id2")
@@ -61,6 +64,7 @@ merge_ids.default <- function(id1, id2, tie_sort = NULL,
   pr_match <- !(!duplicated(id1, fromLast = TRUE) & !duplicated(id1, fromLast = FALSE))
 
   if(isTRUE(shrink)){
+    # id1[!pr_match] <- max(c(id1, id2)) + 1L
     new_id <- combi(id1, id2)
     if(is.null(tie_sort)){
       ord <- order(new_id, sn1)
@@ -97,7 +101,7 @@ merge_ids.default <- function(id1, id2, tie_sort = NULL,
     }
     repo$cr_match <- !(!duplicated(repo$id2, fromLast = TRUE) & !duplicated(repo$id2, fromLast = FALSE))
     repo$new_id <- repo$id1
-    lgk <- (repo$tr_pr_match & !repo$pr_match) |
+    lgk <- (repo$tr_pr_match & (!repo$pr_match | overwrite)) |
       (!repo$tr_pr_match & repo$cr_match)
     repo$new_id[lgk] <- repo$tr_id1[lgk]
   }
@@ -124,9 +128,10 @@ merge_ids.pid <- function(id1, id2, tie_sort = NULL,
   new_pid@.Data <- merge_ids.default(id1 = id1@.Data,
                                      id2 = id2@.Data,
                                      tie_sort = tie_sort)
-  new_pid@link_id <- merge_ids.default(id1 = id1@link_id,
-                                       id2 = id2@link_id,
-                                       tie_sort = tie_sort)
+  new_pid@link_id <- list(link_id1 = new_pid@link_id)
+  # new_pid@link_id <- merge_ids.default(id1 = id1@link_id,
+  #                                      id2 = id2@link_id,
+  #                                      tie_sort = tie_sort)
   lgk <- id1@.Data != new_pid@.Data
   lgk <- new_pid@.Data %in% new_pid@.Data[lgk]
   mx_cri <- max(id1@pid_cri)
